# NEST Firmware - Arduino Version (Prototipo Wearable)

Este repositorio contiene el firmware para el proyecto **NEST (Non-intrusive dEviceS for Telemedicine)**, desarrollado para un **prototipo wearable** en la plataforma **Arduino**. El firmware controla el monitoreo de **frecuencia cardíaca (BPM)** mediante el sensor **MAX30102** y el microcontrolador **nRF52840** con **Bluetooth Low Energy (BLE)** integrado. Este repositorio incluye el bootloader y las bibliotecas necesarias para compilar y programar el código.

## Estructura del repositorio

- **/bootloader**: Archivos del bootloader para el nRF52840.
- **/firmware**: Código fuente del firmware desarrollado en Arduino.
- **/libraries**: Biblioteca necesaria, como **Ucglib.h**, utilizada para controlar la pantalla TFT.

## Requisitos para Programar en Arduino

### Instalar Arduino Core para nRF52840

1. **Descargar e instalar Arduino IDE** (al menos v1.6.12): [Arduino IDE](https://www.arduino.cc/en/software).
2. Iniciar el Arduino IDE y entrar a las **Preferencias**.
3. En **Additional Board Manager URLs**, agregar:  
   `https://adafruit.github.io/arduino-board-index/package_adafruit_index.json`
4. Reiniciar el Arduino IDE.
5. Ir a **Herramientas > Placa > Gestor de Placas** y buscar "Adafruit nRF52" e instalar **Adafruit nRF52 by Adafruit**.
6. Una vez instalado, seleccionar **Adafruit Feather nRF52840 Express** desde **Herramientas > Placa** para configurar el sistema.


## Bootloader

El bootloader puede actualizarse mediante un archivo UF2 o DFU si ya existe uno. También es posible cargarlo en un chip nuevo en blanco siguiendo la guía a continuación.

### Actualización del Bootloader con UF2 (Solo para nRF52840 y requiere versión 0.4.0+)

Esto solo funciona con nRF52840 y requiere que el bootloader existente tenga al menos la versión 0.4.0:

1. Haz doble clic rápidamente en el botón de reinicio para poner la placa en modo bootloader. Aparecerá un dispositivo de almacenamiento masivo llamado `FTHR840BOOT`.
2. Descarga el último archivo UF2 para tu placa, por ejemplo, `update-{BOARD}-{version}_nosd.uf2` desde la [página de lanzamientos del Bootloader Adafruit nRF52](https://github.com/adafruit/Adafruit_nRF52_Bootloader/releases).
3. Arrastra y suelta el archivo UF2 en la unidad `FTHR840BOOT` para realizar la actualización, luego espera hasta que la placa se reinicie automáticamente.

### Actualización del Bootloader con DFU

Para actualizar al último Bootloader + Softdevice usando el puerto serie desde el IDE de Arduino:

1. Selecciona **Herramientas > Placa > Adafruit Feather nRF52840 Express**.
2. Selecciona **Herramientas > Programador > Bootloader DFU for Bluefruit nRF52**.
3. Selecciona **Herramientas > Grabar Bootloader**.
4. **ESPERA** hasta que el proceso termine (~30 segundos).

**Nota:** Cierra el Monitor Serial antes de hacer clic en "Grabar Bootloader". Después, no debes cerrar el IDE de Arduino, desconectar la placa, abrir el Monitor Serial, etc., ya que hay un alto riesgo de dañar el dispositivo. Haz esto con precaución.

### Grabar un nuevo Bootloader

Para grabar el bootloader desde el IDE de Arduino, necesitarás tener instaladas las siguientes herramientas en tu sistema y que estén disponibles en la ruta del sistema:

- [Segger JLink Software and Documentation Pack](https://www.segger.com/downloads/jlink)
- [Nordic nRF5x Command Line Tools](https://www.nordicsemi.com/Software-and-Tools/Development-Tools/nRF-Command-Line-Tools)

Verifica que puedas ejecutar `nrfjprog` desde tu terminal o símbolo del sistema.

**Nota para macOS:** Actualmente, necesitarás crear un enlace simbólico en `/usr/local/bin` hacia la herramienta `nrfjprog` en la ubicación donde la hayas instalado. Puedes ejecutar el siguiente comando:

```bash
$ ln -s $HOME/prog/nordic/nrfjprog/nrfjprog /usr/local/bin/nrfjprog

Una vez que las herramientas estén instaladas y agregadas a la ruta de tu sistema, desde el IDE de Arduino:

1. Selecciona **Herramientas > Placa > Adafruit Feather nRF52840 Express**.
2. Selecciona **Herramientas > Programador > J-Link for Feather52**.
3. Selecciona **Herramientas > Grabar Bootloader** con la placa y el J-Link conectados.

Si deseas modificar el bootloader para tus propias necesidades, puedes consultar su repositorio [Adafruit_nRF52_Bootloader](https://github.com/adafruit/Adafruit_nRF52_Bootloader).

#### Grabar el Bootloader manualmente con nrfjprog

El archivo hex del bootloader se puede encontrar en `bin/bootloader`. Ejecuta el siguiente comando:

```bash
$ nrfjprog -e -f nrf52
$ nrfjprog --program feather_nrf52840_bootloader.hex -f nrf52
$ nrfjprog --reset -f nrf52

## Descripción del Firmware

El firmware permite las siguientes funcionalidades:
- Monitoreo de **BPM** y **SpO2** utilizando el sensor **MAX30102**.
- Interacción con el usuario a través de una pantalla **TFT** controlada con la biblioteca **Ucglib.h**.
- Transmisión de datos a una base de datos remota utilizando **Bluetooth Low Energy (BLE)**.
- Gestión de encuestas de dolor y recordatorios de medicación.


